@@ -70,7 +70,6 @@ class const_iterator;
 class const_preorder_iterator;
 class const_postorder_iterator;
 
-
 /** Lightweight wrapper for GiNaC's symbolic objects.  It holds a pointer to
  *  the other object in order to do garbage collection by the method of
  *  reference counting.  I.e., it is a smart pointer.  Also, the constructor
@@ -127,7 +126,7 @@ public:
 
 	// evaluation
 	ex eval(int level = 0) const { return bp->eval(level); }
-	ex evalf(int level = 0, PyObject* parent=NULL) const 
+	ex evalf(int level = 0, PyObject* parent=nullptr) const 
 	{ return bp->evalf(level, parent); }
 	ex evalm() const { return bp->evalm(); }
 	ex eval_ncmul(const exvector & v) const { return bp->eval_ncmul(v); }
@@ -221,6 +220,9 @@ public:
 	ex simplify_indexed(unsigned options = 0) const;
 	ex simplify_indexed(const scalar_products & sp, unsigned options = 0) const;
 
+	// domains
+	void set_domain(unsigned d);
+
 	// comparison
 	int compare(const ex & other) const;
 	bool is_equal(const ex & other) const;
@@ -322,53 +324,6 @@ ex::ex(const std::string &s, const ex &l) : bp(construct_from_string_and_lst(s,
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
 }
 
-inline
-int ex::compare(const ex & other) const
-{
-#ifdef GINAC_COMPARE_STATISTICS
-	compare_statistics.total_compares++;
-#endif
-	if (bp == other.bp)  // trivial case: both expressions point to same basic
-		return 0;
-#ifdef GINAC_COMPARE_STATISTICS
-	compare_statistics.nontrivial_compares++;
-#endif
-	const int cmpval = bp->compare(*other.bp);
-#if 1
-	if (cmpval == 0) {
-		// Expressions point to different, but equal, trees: conserve
-		// memory and make subsequent compare() operations faster by
-		// making both expressions point to the same tree.
-		share(other);
-	}
-#endif
-	return cmpval;
-}
-
-inline
-bool ex::is_equal(const ex & other) const
-{
-#ifdef GINAC_COMPARE_STATISTICS
-	compare_statistics.total_is_equals++;
-#endif
-	if (bp == other.bp)  // trivial case: both expressions point to same basic
-		return true;
-#ifdef GINAC_COMPARE_STATISTICS
-	compare_statistics.nontrivial_is_equals++;
-#endif
-	const bool equal = bp->is_equal(*other.bp);
-#if 0
-	if (equal) {
-		// Expressions point to different, but equal, trees: conserve
-		// memory and make subsequent compare() operations faster by
-		// making both expressions point to the same tree.
-		share(other);
-	}
-#endif
-	return equal;
-}
-
-
 // Iterators
 
 class const_iterator : public std::iterator<std::random_access_iterator_tag, ex, ptrdiff_t, const ex *, const ex &> {
@@ -380,7 +335,7 @@ public:
 	const_iterator() throw() {}
 
 private:
-	const_iterator(const ex &e_, size_t i_) throw() : e(e_), i(i_) {}
+	const_iterator(ex e_, size_t i_) throw() : e(std::move(e_)), i(i_) {}
 
 public:
 	// This should return an ex&, but that would be a reference to a
@@ -392,9 +347,9 @@ public:
 
 	// This should return an ex*, but that would be a pointer to a
 	// temporary value
-	std::auto_ptr<ex> operator->() const
+	std::unique_ptr<ex> operator->() const
 	{
-		return std::auto_ptr<ex>(new ex(operator*()));
+		return std::unique_ptr<ex>(new ex(operator*()));
 	}
 
 	ex operator[](difference_type n) const
@@ -498,7 +453,7 @@ protected:
 namespace internal {
 
 struct _iter_rep {
-	_iter_rep(const ex &e_, size_t i_, size_t i_end_) : e(e_), i(i_), i_end(i_end_) {}
+	_iter_rep(ex e_, size_t i_, size_t i_end_) : e(std::move(e_)), i(i_), i_end(i_end_) {}
 
 	bool operator==(const _iter_rep &other) const throw()
 	{
@@ -777,7 +732,7 @@ inline ex collect(const ex & thisex, const ex & s, bool distributed = false)
 inline ex eval(const ex & thisex, int level = 0)
 { return thisex.eval(level); }
 
-inline ex evalf(const ex & thisex, int level = 0, PyObject* parent=NULL)
+inline ex evalf(const ex & thisex, int level = 0, PyObject* parent=nullptr)
 { return thisex.evalf(level, parent); }
 
 inline ex evalm(const ex & thisex)
